from blaugue.base import BlaugueBase


class BlaugueBot(BlaugueBase):

    def __init__(self, **bot):
        super().__init__(bot['name'], bot['token'])
        self.base = super()
        self.bot_name = bot['name']
        self.bot_token = bot['token']




from blaugue.user import BlaugueUser
from blaugue.base import BlaugueBase
import markdown


class BlauguePost:

    def __init__(self, infos: dict, base: BlaugueBase):
        self.infos = infos
        self.base = base

    @property
    def title(self):
        return self.infos['titre']

    @property
    def subtitle(self):
        return self.infos['commentaire']

    @property
    def html_content(self):
        return markdown.markdown(self.infos['contenu'])

    @property
    def markdown_content(self):
        return self.infos['contenu']

    @property
    def image(self):
        return self.infos['image']

    @property
    def book(self):
        return self.infos['book']

    @property
    def id(self):
        return self.infos['_id']

    @property
    def url(self):
        return 'http://blaugue.camponovo.xyz/post/' + self.infos['_id']

    @property
    def author(self):
        return BlaugueUser(self.base.request(self.base.user_url.format(self.infos['user'])).json(), self.base)

    @property
    def type(self):
        return self.infos['type']

    @property
    def comments(self):
        return self.infos['comments']





class BlaugueUser:

    def __init__(self, infos: dict, base: object):
        self.infos = infos
        self.base = base

    @property
    def name(self):
        return self.infos['name']

    @property
    def id(self):
        return self.infos['_id']

    @property
    def avatar(self):
        return self.base.baseurl + self.infos['pp'] if self.infos['pp'] is not None else 'No avatar'

    @property
    def premium(self):
        return self.infos['premium']

    @property
    def premium_icon(self):
        return self.base.baseurl + self.infos['prem_url']

    @property
    def badges(self):
        return self.infos['badges'].split(', ')

    @property
    def description(self):
        return self.infos['desc']

    @property
    def url(self):
        return 'http://blaugue.camponovo.xyz/user/' + self.infos['name']

    @property
    def bot(self):
        return self.infos['bot']



from blaugue.post import BlauguePost


class BlaugueTag:

    def __init__(self, tag: dict, base: object):
        self.tag = tag
        self.base = base

    @property
    def name(self):
        return self.tag['type']

    @property
    def posts(self):
        return [BlauguePost(tag, self.base) for tag in self.base.request(self.base.search_url.format(self.name)).json()['tags']]

    @property
    def url(self):
        return 'http://blaugue.camponovo.xyz/type/' + self.name



from blaugue.api import Blaugue
from blaugue.post import BlauguePost
from blaugue.user import BlaugueUser
from blaugue.tag import BlaugueTag
from blaugue.search import BlaugueResults
from blaugue.chat import BlaugueChat, BlaugueMessage
from blaugue import post, tag, user, search, chat

__version__ = '0.0.1'



from blaugue.user import BlaugueUser
from blaugue.post import BlauguePost
from blaugue.tag import BlaugueTag


class BlaugueResults:

    def __init__(self, results: dict, base: object):
        results['tags'] = [BlaugueTag(tag, base) for tag in results['tags']]
        results['users'] = [BlaugueUser(user, base) for user in results['users']]
        results['posts'] = [BlauguePost(post, base) for post in results['posts']]
        self._results = results

    @property
    def results(self):
        """
        :return: List of all results
        """
        return self._results['users'] + self._results['posts'] + self._results['tags']

    @property
    def json_results(self):
        """
        :return: List of all results
        """
        return self._results

    @property
    def users_results(self):
        """
        :return: List of BlaugueUser() object
        """
        return self._results['users']

    @property
    def posts_results(self):
        """
        :return: List of BlauguePost() object
        """
        return self._results['posts']

    @property
    def tags_results(self):
        """
        :return: List of BlaugueTag() object
        """
        return self._results['tags']



import requests


class BlaugueBase:

    def __init__(self, user, password):
        self.base_url = 'http://127.0.0.1:2000'
        # self.base_url = 'http://blaugue.camponovo.xyz'
        self.api_url = 'http://127.0.0.1:2000/api'
        # self.api_url = 'http://blaugue.camponovo.xyz/api'
        self.headers = {'blaugueathoken': 'no', 'blaugueuser': user}
        self.user = user
        self.password = password

    @property
    def search_url(self):
        return self.api_url + '/search/{}'

    @property
    def user_url(self):
        return self.api_url + '/user/{}'

    @property
    def post_url(self):
        return self.api_url + '/post/{}'

    @property
    def chat_url(self):
        return self.api_url + '/chat/{}'

    @property
    def url(self):
        return self.api_url

    @property
    def baseurl(self):
        return self.base_url

    def set_token(self):
        self.headers['blaugueathoken'] = self.get_token()
        self.headers['blaugueuser'] = self.user

    def isright(self):
        try:
            return self.request(self.api_url + '/auth/verify').json()['account_status']
        except:
            raise BlaugueError('Invalid Account informations !')

    def get_password(self):
        return bytes(self.request(self.api_url + '/get_password').content)

    def get_token(self):
        return self.request(self.api_url + '/auth/' + self.user + '/' + self.password).content.decode()

    def request(self, url):
        return requests.get(url, headers=self.headers)


class BlaugueError(Exception):
    """
    BlaugueError class
    """



from blaugue import user


class BlaugueMessage:

    def __init__(self, infos: dict, base: object, chat):
        self.infos = infos
        self.base = base
        self.chat = chat

    @property
    def html_content(self):
        return self.infos['message']

    @property
    def content(self):
        return self.infos['message'].replace('<br>', '').replace('&nbsp;', '')

    @property
    def author(self):
        return user.BlaugueUser(self.base.request(self.base.user_url.format(self.infos['from']['name'])).json(), self.base)

    @property
    def receiver(self):
        if self.chat.id == 'blaugue':
            return [user.BlaugueUser(info, self.base) for info in self.base.request(self.base.user_url.format(self.infos['to'])).json()['users']]
        return user.BlaugueUser(self.base.request(self.base.user_url.format(self.infos['to'])).json(), self.base)


class BlaugueChat:

    def __init__(self, infos: dict, base: object):
        self.infos = infos
        self.base = base

    @property
    def id(self):
        return self.infos['chatname']

    @property
    def name(self):
        name = self.infos['chatname'].split('@')[0]
        try:
            name = name.split('.')[0]
        finally:
            name = name[0].upper() + name[1:]
        return name

    @property
    def messages(self):
        return [BlaugueMessage(mes, self.base, self) for mes in self.infos['messages']]



from setuptools import setup

VERSION = '0.0.1'
DESCRIPTION = 'Official Blaugue API package'
f = open('README.md')
LONG_DESCRIPTION = f.read()
f.close()

setup(
    name="blaugue",
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    author="CAMARM-DEV",
    author_email="armand@camponovo.xyz",
    license='MIT',
    packages=['blaugue'],
    install_requires=['requests~=2.25.1',
'Markdown~=3.3.4',
'setuptools~=57.0.0'],
    keywords='conversion',
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        'License :: OSI Approved :: MIT License',
        "Programming Language :: Python :: 3",
    ],
    long_description_content_type='text/markdown'
)



from blaugue import search, user, post, chat
from blaugue.base import BlaugueBase


class Blaugue(BlaugueBase):

    def __init__(self, **connexion):
        super().__init__(connexion['email'], connexion['password'])
        self.user = connexion['email']
        self.password = connexion['password']
        super().set_token()

    def search(self, query):
        return search.BlaugueResults(super().request(super().search_url.format(query)).json(), super())

    def get_user(self, email):
        return user.BlaugueUser(super().request(super().user_url.format(email)).json(), super())

    def get_post(self, post_id):
        return post.BlauguePost(super().request(super().post_url.format(post_id)).json(), super())

    def all_users(self):
        all_users = []
        for infos in super().request(super().user_url.format('all')).json()['users']:
            all_users.append(user.BlaugueUser(infos, super()))
        return all_users

    def get_chats(self):
        return [self.get_chat(chat_name['name']) for chat_name in super().request(super().url + '/chats').json()]

    def get_chat(self, chat_name):
        return chat.BlaugueChat(super().request(super().chat_url.format(chat_name)).json(), super())

    def __str__(self):
        return self.user

    def __bool__(self):
        return super().isright()

    def __next__(self):
        return super().get_token()

    def __bytes__(self):
        return super().get_password()

    @property
    def connected(self):
        return self.get_user(self.user)
